module EnumLNP

require 'rubygems'
require 'dnsruby'

#This script will perform DNS query of type NAPTR to route the call properly.
#This script uses the service E2U+sip or E2U+SIP

  $domain = "IPaddress/FQDN"   #This is the domain where the NAPTR requests will be send to.

def trace_log( msg )
  puts "   \t#{msg}" if msg && !msg.empty?
  STDOUT.flush
end
#
# get a syntax taht we can query for NAPTR
# for example:
# IN             OUT
# 5145551234 -> 5.1.4.5.5.5.1.2.3.4
#
def get_naptr_syntax(number)
  number.reverse!.scan(/.{1}|.+/).join(".")
end



#
# Method that will send a NAPTR request to a DNS server.
# It looks for service E2U+pstn:tel or E2U+tel in the NAPTR response.
# Arguments:
#   host: hostname of the DNS server
#   params: call parameters
# Returns:
# tel_called: the new telephone called number proposed by the DNS server
#
#
def tb_naptr_query(host, params)
  begin
  res = Dnsruby::Resolver.new (:nameserver => ['IPaddress'])
    #trace_log "making a NAPTR query to #{host}"
    answer = res.query(host, "NAPTR")
    if (answer.header.ancount < 1)
      print "host #{host} has no NAPTR record\n"
    else
      trace_log "#{host} has #{answer.header.ancount} NAPTR results."
      #trace_log answer.answer.inspect
      answer.answer.each do |a|
        trace_log "answer---> service='#{a.service}' order=#{a.order} pref=#{a.preference} replacement name=#{a.regexp.to_s}"

        if a.service == "sip+E2U" || a.service == "SIP+E2U"
          #extract the called returned by the answer tel:+(FOO)
          tel_called = a.regexp.to_s.scan( /sip:((\D)(\d*))/).last.first
          log_trace :always, "Changing called number from '#{params[:call][:called]}' to '#{tel_called}'"
          #params[:call][:called] = tel_called
          #no need to continue to following records, stop here
          return tel_called
        else
          #trace_log "Ignoring service #{a.service}"
        end
      end
    end
  rescue Dnsruby::NXDomain => e
    trace_log "host #{host} cannot be resolved."
  rescue Exception => e
    trace_log "Unknown Exception occured: #{e}"
  end
end


#
# before filter

def naptr_called_replacement params
  call = params[ :call ]

  #we need to clone call[:called] here, otherwise, we will reverse the original called number.
  called_clone = call[:called].clone

  naptr_called = get_naptr_syntax(called_clone)
  params[:call][:called] = tb_naptr_query("#{naptr_called}@#{$domain}", params)

  params
end


# Filter Initialization method
def init_naptr_called_replacement params
end

end
